;-----------------
function bg_mpfit, a, XVAL=x, YVAL=y, ERRVAL=err, CCMAPS=CCMAPS, BETAVEC=BETAVEC, TDVEC=TDVEC

if not keyword_set(err) then begin
    err = x
    err(*) = 1.
endif

model = bg(x, a)

; colour correction
if (keyword_set(CCMAPS)) then begin
   ii = interpol(indgen(n_elements(tdvec)), tdvec, a[1])
   jj = interpol(indgen(n_elements(betavec)), betavec, a[2])
   kk = indgen(n_elements(x))
   cc = interpolate(ccmaps, ii, jj, kk, /grid)
   model = model*reform(cc)
endif

res = (y-model)/err

return, res

end

;-----------------
function bg_log_mpfit, a, XVAL=x, YVAL=y, ERRVAL=err

if not keyword_set(err) then begin
    err = x
    err(*) = 1.
endif

model = bg(x, a)
res = (alog(y)-alog(model))/alog(err)

return, res

end

;----------------------------------------
function fitbg, lambda, data, aguess, errordata=errordata, verbose=verbose, $
                fixed=fixed, perror=perror, log=log, twobeta=twobeta, chi2=chi2, $
                CCMAPS=CCMAPS, BETAVEC=BETAVEC, TDVEC=TDVEC, status=status

  if not keyword_set(errordata) then errordata = replicate(1, n_elements(data))
  if keyword_set(verbose) then quiet=0 else quiet=1

  ; colour correction keywords
  if not keyword_set(CCMAPS) then CCMAPS=''
  if not keyword_set(BETAVEC) then BETAVEC=''
  if not keyword_set(TDVEC) then TDVEC=''

  if not keyword_set(aguess) then begin
     if keyword_set(twobeta) then aguess=[1., 17.5, 2., 550., 2.] else aguess = [1., 17.5, 2.]
  endif
  if not keyword_set(fixed) then fixed= intarr(n_elements(aguess))

  if keyword_set(log) then fname = 'bg_log_mpfit' else fname = 'bg_mpfit'

  if keyword_set(twobeta) then begin
     parinfo = {value:aguess, $
                fixed:fixed, $
                limited:[[1,0], [1,1], [1,1], [1,1], [1,1]], $
                limits:[[1.e-8,0.], [5.,200.], [1.0,2.2], [300,max(lambda)], [0.1,5.]], $
                step:[0,0,0,0,0]}
  endif else begin
     parinfo = {value:aguess, $
                fixed:fixed, $
                limited:[[1,0], [1,1], [1,1]], $
                limits:[[0,0], [5,200], [0.5,5.0]], $
                step:[0.01,0.01,0.01]}
  endelse

  functargs = {XVAL:lambda, YVAL:data, ERRVAL:errordata, CCMAPS:CCMAPS, BETAVEC:BETAVEC, TDVEC:TDVEC}
  result = mpfit(fname, functargs=functargs, parinfo=parinfo, quiet=quiet, perror=perror, status=status, errmsg=errmsg, $
                FTOL=1e-20, XTOL=1e-10) 

  if (status le 0.) then print, 'status:', status, ' : ', errmsg

  rien = where(fixed eq 0, nbparam)
  nbfree = n_elements(data) - nbparam
  model = bg(lambda, result)

; colour correction
  if keyword_set(tdvec) then begin
     ii = interpol(indgen(n_elements(tdvec)), tdvec, result[1])
     jj = interpol(indgen(n_elements(betavec)), betavec, result[2])
     kk = indgen(n_elements(lambda))
     cc = interpolate(ccmaps, ii, jj, kk, /grid)
     model = model*reform(cc)
  endif

   chi2 = total( (data-model)^2 / errordata^2 ) / nbfree

   return, result

end
